'' =================================================================================================
''
''   File....... jm_ez_sequencer_v2.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 06 OCT 2013
''
'' =================================================================================================


con { speed/timing control }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con { io pins }

  RX1  = 31                                                     ' programming / terminal
  TX1  = 30
  
  SDA  = 29                                                     ' eeprom / i2c
  SCL  = 28

  LED7 = 23                                                     ' leds 
  LED6 = 22
  LED5 = 21
  LED4 = 20
  LED3 = 19
  LED2 = 18
  LED1 = 17
  LED0 = 16


con { brighness levels }

  BR0 =   0
  BR1 =  32
  BR2 =  92
  BR3 = 255
   

obj

  leds : "jm_pwm8"
   

var


pub main

  leds.start(8, LED0)

  repeat
    play_pattern(@WaveL2R, 8, 75, 3, -1)
    pause(1000)
  
    play_pattern(@CylonStart, 8, 150, 1, -1)
    play_pattern(@CylonRun,   8, 125, 5, -1)
    play_pattern(@CylonStop,  8, 150, 1, -1)
    pause(1000)
     

pub play_pattern(p_pat, count, ms, cycles, p_flag) | steps, p_leds, outs, ch, level

'' Play pattern
'' -- p_pat is pointer to pattern
'' -- count is number of channels in pattern (1-8)
'' -- ms is timing (milliseconds) for each step
'' -- cycles is # of times to play pattern
'' -- p_flag is pointer to flag variable (set to false when done)
''    * use -1 for no flag

  count <#= 8                                                   ' limit to 8 channels

  if (p_flag > 0)                                               ' using flag?
    long[p_flag] := true                                        '  mark busy

  repeat cycles
    steps := long[p_pat]                                        ' get steps in pattern
    p_leds := p_pat + 4                                         ' set working pointer to 1st step
    repeat steps                                                ' loop through steps
      outs := long[p_leds]
      repeat ch from 0 to count-1
        level := outs & %11                                     ' extract low quart
        level := lookupz(level: BR0, BR1, BR2, BR3)             ' convert to brightness
        leds.set(ch, level)                                     ' set output
        outs >>= 2                                              ' next quart
      pause(ms)                                                 ' hold step      
      p_leds += 4                                               ' point to next step

  if (p_flag > 0)                                               ' using flag?
    long[p_flag] := false                                       '  mark done


con

  { --------------- }
  {   B A S I C S   }
  { --------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

con

  { ------------------- }
  {  S E Q U E N C E S  }
  { ------------------- }


dat
                        long

  WaveL2R               long    13
                        long    %%1_0_0_0_0_0_0_0
                        long    %%2_1_0_0_0_0_0_0
                        long    %%3_2_1_0_0_0_0_0
                        long    %%2_3_2_1_0_0_0_0
                        long    %%1_2_3_2_1_0_0_0
                        long    %%0_1_2_3_2_1_0_0
                        long    %%0_0_1_2_3_2_1_0
                        long    %%0_0_0_1_2_3_2_1
                        long    %%0_0_0_0_1_2_3_2
                        long    %%0_0_0_0_0_1_2_3
                        long    %%0_0_0_0_0_0_1_2
                        long    %%0_0_0_0_0_0_0_1
                        long    %%0_0_0_0_0_0_0_0
                        
                        
  CylonStart            long    5
                        long    %%0_0_0_0_1_0_0_0
                        long    %%0_0_0_0_2_0_0_0
                        long    %%0_0_0_1_3_1_0_0
                        long    %%0_0_0_1_3_1_0_0
                        long    %%0_0_0_1_3_1_0_0
                        
  CylonRun              long    14
                        long    %%0_0_0_0_1_3_1_0
                        long    %%0_0_0_0_0_1_3_1 
                        long    %%0_0_0_0_0_0_1_3 
                        long    %%0_0_0_0_0_1_3_1 
                        long    %%0_0_0_0_1_3_1_0
                        long    %%0_0_0_1_3_1_0_0
                        long    %%0_0_1_3_1_0_0_0
                        long    %%0_1_3_1_0_0_0_0
                        long    %%1_3_1_0_0_0_0_0
                        long    %%3_1_0_0_0_0_0_0
                        long    %%1_3_1_0_0_0_0_0
                        long    %%0_1_3_1_0_0_0_0  
                        long    %%0_0_1_3_1_0_0_0  
                        long    %%0_0_0_1_3_1_0_0 

  CylonStop             long    6
                        long    %%0_0_0_1_3_1_0_0 
                        long    %%0_0_0_1_3_1_0_0
                        long    %%0_0_0_1_3_1_0_0
                        long    %%0_0_0_0_2_0_0_0  
                        long    %%0_0_0_0_1_0_0_0
                        long    %%0_0_0_0_0_0_0_0
                                               

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}